@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 # Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
@@ -12,8 +10,11 @@ import os
 import sys
 import time
 
-from psutil._common import sdiskusage, usage_percent, memoize
-from psutil._compat import PY3, unicode
+from ._common import memoize
+from ._common import sdiskusage
+from ._common import usage_percent
+from ._compat import PY3
+from ._compat import unicode
 
 
 class TimeoutExpired(Exception):
@@ -31,8 +32,7 @@ def pid_exists(pid):
         return True
     try:
         os.kill(pid, 0)
-    except OSError:
-        err = sys.exc_info()[1]
+    except OSError as err:
         if err.errno == errno.ESRCH:
             # ESRCH == No such process
             return False
@@ -69,17 +69,18 @@ def wait_pid(pid, timeout=None):
 
     timer = getattr(time, 'monotonic', time.time)
     if timeout is not None:
-        waitcall = lambda: os.waitpid(pid, os.WNOHANG)
+        def waitcall():
+            return os.waitpid(pid, os.WNOHANG)
         stop_at = timer() + timeout
     else:
-        waitcall = lambda: os.waitpid(pid, 0)
+        def waitcall():
+            return os.waitpid(pid, 0)
 
     delay = 0.0001
-    while 1:
+    while True:
         try:
             retpid, status = waitcall()
-        except OSError:
-            err = sys.exc_info()[1]
+        except OSError as err:
             if err.errno == errno.EINTR:
                 delay = check_timeout(delay)
                 continue
@@ -90,7 +91,7 @@ def wait_pid(pid, timeout=None):
                 # - pid never existed in the first place
                 # In both cases we'll eventually return None as we
                 # can't determine its exit status code.
-                while 1:
+                while True:
                     if pid_exists(pid):
                         delay = check_timeout(delay)
                     else:
@@ -123,7 +124,7 @@ def disk_usage(path):
         if not PY3 and isinstance(path, unicode):
             # this is a bug with os.statvfs() and unicode on
             # Python 2, see:
-            # - https://code.google.com/p/psutil/issues/detail?id=416
+            # - https://github.com/giampaolo/psutil/issues/416
             # - http://bugs.python.org/issue18695
             try:
                 path = path.encode(sys.getfilesystemencoding())
@@ -150,8 +151,7 @@ def _get_terminal_map():
         assert name not in ret
         try:
             ret[os.stat(name).st_rdev] = name
-        except OSError:
-            err = sys.exc_info()[1]
+        except OSError as err:
             if err.errno != errno.ENOENT:
                 raise
     return ret
