@@ -4,20 +4,31 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-# TODO: add test for comparing connections with 'sockstat' cmd
+# TODO: (FreeBSD) add test for comparing connections with 'sockstat' cmd.
 
-"""BSD specific tests.  These are implicitly run by test_psutil.py."""
+"""Tests specific to all BSD platforms.  These are implicitly run by
+test_psutil.
+py."""
 
-import subprocess
-import time
-import sys
+import datetime
 import os
+import subprocess
+import sys
+import time
 
 import psutil
-
 from psutil._compat import PY3
-from test_psutil import (TOLERANCE, sh, get_test_subprocess, which,
-                         retry_before_failing, reap_children, unittest)
+from test_psutil import BSD
+from test_psutil import FREEBSD
+from test_psutil import get_test_subprocess
+from test_psutil import MEMORY_TOLERANCE
+from test_psutil import NETBSD
+from test_psutil import OPENBSD
+from test_psutil import reap_children
+from test_psutil import retry_before_failing
+from test_psutil import sh
+from test_psutil import unittest
+from test_psutil import which
 
 
 PAGESIZE = os.sysconf("SC_PAGE_SIZE")
@@ -32,7 +43,10 @@ def sysctl(cmdline):
     returning only the value of interest.
     """
     result = sh("sysctl " + cmdline)
-    result = result[result.find(": ") + 2:]
+    if FREEBSD:
+        result = result[result.find(": ") + 2:]
+    elif OPENBSD or NETBSD:
+        result = result[result.find("=") + 1:]
     try:
         return int(result)
     except ValueError:
@@ -50,21 +64,23 @@ def muse(field):
     return int(line.split()[1])
 
 
+# =====================================================================
+# --- All BSD*
+# =====================================================================
+
+
+@unittest.skipUnless(BSD, "not a BSD system")
 class BSDSpecificTestCase(unittest.TestCase):
+    """Generic tests common to all BSD variants."""
 
-    def setUp(self):
-        self.pid = get_test_subprocess().pid
+    @classmethod
+    def setUpClass(cls):
+        cls.pid = get_test_subprocess().pid
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         reap_children()
 
-    def test_boot_time(self):
-        s = sysctl('sysctl kern.boottime')
-        s = s[s.find(" sec = ") + 7:]
-        s = s[:s.find(',')]
-        btime = int(s)
-        self.assertEqual(btime, psutil.boot_time())
-
     def test_process_create_time(self):
         cmdline = "ps -o lstart -p %s" % self.pid
         p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
@@ -104,6 +120,39 @@ class BSDSpecificTestCase(unittest.TestCase):
             if abs(usage.used - used) > 10 * 1024 * 1024:
                 self.fail("psutil=%s, df=%s" % (usage.used, used))
 
+    def test_cpu_count_logical(self):
+        syst = sysctl("hw.ncpu")
+        self.assertEqual(psutil.cpu_count(logical=True), syst)
+
+    def test_virtual_memory_total(self):
+        num = sysctl('hw.physmem')
+        self.assertEqual(num, psutil.virtual_memory().total)
+
+
+# =====================================================================
+# --- FreeBSD
+# =====================================================================
+
+
+@unittest.skipUnless(FREEBSD, "not a FreeBSD system")
+class FreeBSDSpecificTestCase(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        cls.pid = get_test_subprocess().pid
+
+    @classmethod
+    def tearDownClass(cls):
+        reap_children()
+
+    def test_boot_time(self):
+        s = sysctl('sysctl kern.boottime')
+        s = s[s.find(" sec = ") + 7:]
+        s = s[:s.find(',')]
+        btime = int(s)
+        self.assertEqual(btime, psutil.boot_time())
+
+    @retry_before_failing()
     def test_memory_maps(self):
         out = sh('procstat -v %s' % self.pid)
         maps = psutil.Process(self.pid).memory_maps(grouped=False)
@@ -118,104 +167,142 @@ class BSDSpecificTestCase(unittest.TestCase):
             if not map.path.startswith('['):
                 self.assertEqual(fields[10], map.path)
 
-    # --- virtual_memory(); tests against sysctl
+    def test_exe(self):
+        out = sh('procstat -b %s' % self.pid)
+        self.assertEqual(psutil.Process(self.pid).exe(),
+                         out.split('\n')[1].split()[-1])
 
-    def test_vmem_total(self):
-        syst = sysctl("sysctl vm.stats.vm.v_page_count") * PAGESIZE
-        self.assertEqual(psutil.virtual_memory().total, syst)
+    def test_cmdline(self):
+        out = sh('procstat -c %s' % self.pid)
+        self.assertEqual(' '.join(psutil.Process(self.pid).cmdline()),
+                         ' '.join(out.split('\n')[1].split()[2:]))
+
+    def test_uids_gids(self):
+        out = sh('procstat -s %s' % self.pid)
+        euid, ruid, suid, egid, rgid, sgid = out.split('\n')[1].split()[2:8]
+        p = psutil.Process(self.pid)
+        uids = p.uids()
+        gids = p.gids()
+        self.assertEqual(uids.real, int(ruid))
+        self.assertEqual(uids.effective, int(euid))
+        self.assertEqual(uids.saved, int(suid))
+        self.assertEqual(gids.real, int(rgid))
+        self.assertEqual(gids.effective, int(egid))
+        self.assertEqual(gids.saved, int(sgid))
+
+    # --- virtual_memory(); tests against sysctl
 
     @retry_before_failing()
     def test_vmem_active(self):
         syst = sysctl("vm.stats.vm.v_active_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().active, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_inactive(self):
         syst = sysctl("vm.stats.vm.v_inactive_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().inactive, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_wired(self):
         syst = sysctl("vm.stats.vm.v_wire_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().wired, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_cached(self):
         syst = sysctl("vm.stats.vm.v_cache_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().cached, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_free(self):
         syst = sysctl("vm.stats.vm.v_free_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().free, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_buffers(self):
         syst = sysctl("vfs.bufspace")
         self.assertAlmostEqual(psutil.virtual_memory().buffers, syst,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     # --- virtual_memory(); tests against muse
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
-    def test_total(self):
+    def test_muse_vmem_total(self):
         num = muse('Total')
         self.assertEqual(psutil.virtual_memory().total, num)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_active(self):
+    def test_muse_vmem_active(self):
         num = muse('Active')
         self.assertAlmostEqual(psutil.virtual_memory().active, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_inactive(self):
+    def test_muse_vmem_inactive(self):
         num = muse('Inactive')
         self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_wired(self):
+    def test_muse_vmem_wired(self):
         num = muse('Wired')
         self.assertAlmostEqual(psutil.virtual_memory().wired, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_cached(self):
+    def test_muse_vmem_cached(self):
         num = muse('Cache')
         self.assertAlmostEqual(psutil.virtual_memory().cached, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_free(self):
+    def test_muse_vmem_free(self):
         num = muse('Free')
         self.assertAlmostEqual(psutil.virtual_memory().free, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
     @retry_before_failing()
-    def test_buffers(self):
+    def test_muse_vmem_buffers(self):
         num = muse('Buffer')
         self.assertAlmostEqual(psutil.virtual_memory().buffers, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
 
-def test_main():
+# =====================================================================
+# --- OpenBSD
+# =====================================================================
+
+
+@unittest.skipUnless(OPENBSD, "not an OpenBSD system")
+class OpenBSDSpecificTestCase(unittest.TestCase):
+
+    def test_boot_time(self):
+        s = sysctl('kern.boottime')
+        sys_bt = datetime.datetime.strptime(s, "%a %b %d %H:%M:%S %Y")
+        psutil_bt = datetime.datetime.fromtimestamp(psutil.boot_time())
+        self.assertEqual(sys_bt, psutil_bt)
+
+
+def main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(BSDSpecificTestCase))
+    if FREEBSD:
+        test_suite.addTest(unittest.makeSuite(FreeBSDSpecificTestCase))
+    elif OPENBSD:
+        test_suite.addTest(unittest.makeSuite(OpenBSDSpecificTestCase))
     result = unittest.TextTestRunner(verbosity=2).run(test_suite)
     return result.wasSuccessful()
 
 if __name__ == '__main__':
-    if not test_main():
+    if not main():
         sys.exit(1)
