@@ -15,8 +15,14 @@ import time
 import psutil
 
 from psutil._compat import PY3
-from test_psutil import (TOLERANCE, sh, get_test_subprocess, reap_children,
-                         retry_before_failing, unittest)
+from test_psutil import get_test_subprocess
+from test_psutil import MEMORY_TOLERANCE
+from test_psutil import OSX
+from test_psutil import reap_children
+from test_psutil import retry_before_failing
+from test_psutil import sh
+from test_psutil import TRAVIS
+from test_psutil import unittest
 
 
 PAGESIZE = os.sysconf("SC_PAGE_SIZE")
@@ -47,12 +53,42 @@ def vm_stat(field):
     return int(re.search('\d+', line).group(0)) * PAGESIZE
 
 
+# http://code.activestate.com/recipes/578019/
+def human2bytes(s):
+    SYMBOLS = {
+        'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
+    }
+    init = s
+    num = ""
+    while s and s[0:1].isdigit() or s[0:1] == '.':
+        num += s[0]
+        s = s[1:]
+    num = float(num)
+    letter = s.strip()
+    for name, sset in SYMBOLS.items():
+        if letter in sset:
+            break
+    else:
+        if letter == 'k':
+            sset = SYMBOLS['customary']
+            letter = letter.upper()
+        else:
+            raise ValueError("can't interpret %r" % init)
+    prefix = {sset[0]: 1}
+    for i, s in enumerate(sset[1:]):
+        prefix[s] = 1 << (i+1)*10
+    return int(num * prefix[letter])
+
+
+@unittest.skipUnless(OSX, "not an OSX system")
 class OSXSpecificTestCase(unittest.TestCase):
 
-    def setUp(self):
-        self.pid = get_test_subprocess().pid
+    @classmethod
+    def setUpClass(cls):
+        cls.pid = get_test_subprocess().pid
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         reap_children()
 
     def test_process_create_time(self):
@@ -94,35 +130,44 @@ class OSXSpecificTestCase(unittest.TestCase):
             if abs(usage.used - used) > 10 * 1024 * 1024:
                 self.fail("psutil=%s, df=%s" % usage.used, used)
 
+    def test_cpu_count_logical(self):
+        num = sysctl("sysctl hw.logicalcpu")
+        self.assertEqual(num, psutil.cpu_count(logical=True))
+
+    def test_cpu_count_physical(self):
+        num = sysctl("sysctl hw.physicalcpu")
+        self.assertEqual(num, psutil.cpu_count(logical=False))
+
     # --- virtual mem
 
     def test_vmem_total(self):
         sysctl_hwphymem = sysctl('sysctl hw.memsize')
         self.assertEqual(sysctl_hwphymem, psutil.virtual_memory().total)
 
+    @unittest.skipIf(TRAVIS, "")
     @retry_before_failing()
     def test_vmem_free(self):
         num = vm_stat("free")
         self.assertAlmostEqual(psutil.virtual_memory().free, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_active(self):
         num = vm_stat("active")
         self.assertAlmostEqual(psutil.virtual_memory().active, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_inactive(self):
         num = vm_stat("inactive")
         self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     @retry_before_failing()
     def test_vmem_wired(self):
         num = vm_stat("wired")
         self.assertAlmostEqual(psutil.virtual_memory().wired, num,
-                               delta=TOLERANCE)
+                               delta=MEMORY_TOLERANCE)
 
     # --- swap mem
 
@@ -135,23 +180,22 @@ class OSXSpecificTestCase(unittest.TestCase):
         self.assertEqual(psutil.swap_memory().sout, num)
 
     def test_swapmem_total(self):
-        tot1 = psutil.swap_memory().total
-        tot2 = 0
-        # OSX uses multiple cache files:
-        # http://en.wikipedia.org/wiki/Paging#OS_X
-        for name in os.listdir("/var/vm/"):
-            file = os.path.join("/var/vm", name)
-            if os.path.isfile(file):
-                tot2 += os.path.getsize(file)
-        self.assertEqual(tot1, tot2)
+        out = sh('sysctl vm.swapusage')
+        out = out.replace('vm.swapusage: ', '')
+        total, used, free = re.findall('\d+.\d+\w', out)
+        psutil_smem = psutil.swap_memory()
+        self.assertEqual(psutil_smem.total, human2bytes(total))
+        self.assertEqual(psutil_smem.used, human2bytes(used))
+        self.assertEqual(psutil_smem.free, human2bytes(free))
 
 
-def test_main():
+def main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(OSXSpecificTestCase))
     result = unittest.TextTestRunner(verbosity=2).run(test_suite)
     return result.wasSuccessful()
 
+
 if __name__ == '__main__':
-    if not test_main():
+    if not main():
         sys.exit(1)
