@@ -11,12 +11,24 @@ import os
 import subprocess
 import sys
 import time
-import unittest
 
 import psutil
-
-from psutil._compat import PY3
-from test_psutil import *
+from psutil._compat import PY3, callable
+from test_psutil import BSD
+from test_psutil import get_kernel_version
+from test_psutil import get_test_subprocess
+from test_psutil import LINUX
+from test_psutil import OSX
+from test_psutil import POSIX
+from test_psutil import PYTHON
+from test_psutil import reap_children
+from test_psutil import retry_before_failing
+from test_psutil import sh
+from test_psutil import skip_on_access_denied
+from test_psutil import SUNOS
+from test_psutil import TRAVIS
+from test_psutil import unittest
+from test_psutil import wait_for_pid
 
 
 def ps(cmd):
@@ -40,78 +52,89 @@ def ps(cmd):
         return output
 
 
+@unittest.skipUnless(POSIX, "not a POSIX system")
 class PosixSpecificTestCase(unittest.TestCase):
     """Compare psutil results against 'ps' command line utility."""
 
-    # for ps -o arguments see: http://unixhelp.ed.ac.uk/CGI/man-cgi?ps
-    def setUp(self):
-        self.pid = get_test_subprocess([PYTHON, "-E", "-O"],
-                                       stdin=subprocess.PIPE).pid
+    @classmethod
+    def setUpClass(cls):
+        cls.pid = get_test_subprocess([PYTHON, "-E", "-O"],
+                                      stdin=subprocess.PIPE).pid
+        wait_for_pid(cls.pid)
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         reap_children()
 
+    # for ps -o arguments see: http://unixhelp.ed.ac.uk/CGI/man-cgi?ps
+
     def test_process_parent_pid(self):
         ppid_ps = ps("ps --no-headers -o ppid -p %s" % self.pid)
-        ppid_psutil = psutil.Process(self.pid).ppid
+        ppid_psutil = psutil.Process(self.pid).ppid()
         self.assertEqual(ppid_ps, ppid_psutil)
 
     def test_process_uid(self):
         uid_ps = ps("ps --no-headers -o uid -p %s" % self.pid)
-        uid_psutil = psutil.Process(self.pid).uids.real
+        uid_psutil = psutil.Process(self.pid).uids().real
         self.assertEqual(uid_ps, uid_psutil)
 
     def test_process_gid(self):
         gid_ps = ps("ps --no-headers -o rgid -p %s" % self.pid)
-        gid_psutil = psutil.Process(self.pid).gids.real
+        gid_psutil = psutil.Process(self.pid).gids().real
         self.assertEqual(gid_ps, gid_psutil)
 
     def test_process_username(self):
         username_ps = ps("ps --no-headers -o user -p %s" % self.pid)
-        username_psutil = psutil.Process(self.pid).username
+        username_psutil = psutil.Process(self.pid).username()
         self.assertEqual(username_ps, username_psutil)
 
     @skip_on_access_denied()
+    @retry_before_failing()
     def test_process_rss_memory(self):
         # give python interpreter some time to properly initialize
         # so that the results are the same
         time.sleep(0.1)
         rss_ps = ps("ps --no-headers -o rss -p %s" % self.pid)
-        rss_psutil = psutil.Process(self.pid).get_memory_info()[0] / 1024
+        rss_psutil = psutil.Process(self.pid).memory_info()[0] / 1024
         self.assertEqual(rss_ps, rss_psutil)
 
     @skip_on_access_denied()
+    @retry_before_failing()
     def test_process_vsz_memory(self):
         # give python interpreter some time to properly initialize
         # so that the results are the same
         time.sleep(0.1)
         vsz_ps = ps("ps --no-headers -o vsz -p %s" % self.pid)
-        vsz_psutil = psutil.Process(self.pid).get_memory_info()[1] / 1024
+        vsz_psutil = psutil.Process(self.pid).memory_info()[1] / 1024
         self.assertEqual(vsz_ps, vsz_psutil)
 
     def test_process_name(self):
         # use command + arg since "comm" keyword not supported on all platforms
-        name_ps = ps("ps --no-headers -o command -p %s" % self.pid).split(' ')[0]
+        name_ps = ps("ps --no-headers -o command -p %s" % (
+            self.pid)).split(' ')[0]
         # remove path if there is any, from the command
         name_ps = os.path.basename(name_ps).lower()
-        name_psutil = psutil.Process(self.pid).name.lower()
+        name_psutil = psutil.Process(self.pid).name().lower()
         self.assertEqual(name_ps, name_psutil)
 
     @unittest.skipIf(OSX or BSD,
                      'ps -o start not available')
     def test_process_create_time(self):
         time_ps = ps("ps --no-headers -o start -p %s" % self.pid).split(' ')[0]
-        time_psutil = psutil.Process(self.pid).create_time
-        if SUNOS:
-            time_psutil = round(time_psutil)
+        time_psutil = psutil.Process(self.pid).create_time()
         time_psutil_tstamp = datetime.datetime.fromtimestamp(
             time_psutil).strftime("%H:%M:%S")
-        self.assertEqual(time_ps, time_psutil_tstamp)
+        # sometimes ps shows the time rounded up instead of down, so we check
+        # for both possible values
+        round_time_psutil = round(time_psutil)
+        round_time_psutil_tstamp = datetime.datetime.fromtimestamp(
+            round_time_psutil).strftime("%H:%M:%S")
+        self.assertIn(time_ps, [time_psutil_tstamp, round_time_psutil_tstamp])
 
     def test_process_exe(self):
         ps_pathname = ps("ps --no-headers -o command -p %s" %
                          self.pid).split(' ')[0]
-        psutil_pathname = psutil.Process(self.pid).exe
+        psutil_pathname = psutil.Process(self.pid).exe()
         try:
             self.assertEqual(ps_pathname, psutil_pathname)
         except AssertionError:
@@ -126,22 +149,28 @@ class PosixSpecificTestCase(unittest.TestCase):
 
     def test_process_cmdline(self):
         ps_cmdline = ps("ps --no-headers -o command -p %s" % self.pid)
-        psutil_cmdline = " ".join(psutil.Process(self.pid).cmdline)
+        psutil_cmdline = " ".join(psutil.Process(self.pid).cmdline())
         if SUNOS:
             # ps on Solaris only shows the first part of the cmdline
             psutil_cmdline = psutil_cmdline.split(" ")[0]
         self.assertEqual(ps_cmdline, psutil_cmdline)
 
+    def test_process_nice(self):
+        ps_nice = ps("ps --no-headers -o nice -p %s" % self.pid)
+        psutil_nice = psutil.Process().nice()
+        self.assertEqual(ps_nice, psutil_nice)
+
     @retry_before_failing()
-    def test_get_pids(self):
+    def test_pids(self):
         # Note: this test might fail if the OS is starting/killing
         # other processes in the meantime
         if SUNOS:
-            cmd = ["ps", "ax"]
+            cmd = ["ps", "-A", "-o", "pid"]
         else:
             cmd = ["ps", "ax", "-o", "pid"]
         p = get_test_subprocess(cmd, stdout=subprocess.PIPE)
         output = p.communicate()[0].strip()
+        assert p.poll() == 0
         if PY3:
             output = str(output, sys.stdout.encoding)
         pids_ps = []
@@ -151,7 +180,7 @@ class PosixSpecificTestCase(unittest.TestCase):
                 pids_ps.append(pid)
         # remove ps subprocess pid which is supposed to be dead in meantime
         pids_ps.remove(p.pid)
-        pids_psutil = psutil.get_pid_list()
+        pids_psutil = psutil.pids()
         pids_ps.sort()
         pids_psutil.sort()
 
@@ -164,9 +193,10 @@ class PosixSpecificTestCase(unittest.TestCase):
                          [x for x in pids_ps if x not in pids_psutil]
             self.fail("difference: " + str(difference))
 
-    # for some reason ifconfig -a does not report differente interfaces
-    # psutil does
+    # for some reason ifconfig -a does not report all interfaces
+    # returned by psutil
     @unittest.skipIf(SUNOS, "test not reliable on SUNOS")
+    @unittest.skipIf(TRAVIS, "test not reliable on Travis")
     def test_nic_names(self):
         p = subprocess.Popen("ifconfig -a", shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0].strip()
@@ -177,15 +207,18 @@ class PosixSpecificTestCase(unittest.TestCase):
                 if line.startswith(nic):
                     break
             else:
-                self.fail("couldn't find %s nic in 'ifconfig -a' output" % nic)
+                self.fail(
+                    "couldn't find %s nic in 'ifconfig -a' output\n%s" % (
+                        nic, output))
 
-    def test_get_users(self):
+    @retry_before_failing()
+    def test_users(self):
         out = sh("who")
         lines = out.split('\n')
         users = [x.split()[0] for x in lines]
-        self.assertEqual(len(users), len(psutil.get_users()))
+        self.assertEqual(len(users), len(psutil.users()))
         terminals = [x.split()[1] for x in lines]
-        for u in psutil.get_users():
+        for u in psutil.users():
             self.assertTrue(u.name in users, u.name)
             self.assertTrue(u.terminal in terminals, u.terminal)
 
@@ -196,28 +229,29 @@ class PosixSpecificTestCase(unittest.TestCase):
             args = ()
             attr = getattr(p, name, None)
             if attr is not None and callable(attr):
-                if name == 'get_rlimit':
+                if name == 'rlimit':
                     args = (psutil.RLIMIT_NOFILE,)
-                elif name == 'set_rlimit':
-                    args = (psutil.RLIMIT_NOFILE, (5, 5))
-                ret = attr(*args)
+                attr(*args)
             else:
-                ret = attr
+                attr
 
         p = psutil.Process(os.getpid())
         failures = []
+        ignored_names = ['terminate', 'kill', 'suspend', 'resume', 'nice',
+                         'send_signal', 'wait', 'children', 'as_dict']
+        if LINUX and get_kernel_version() < (2, 6, 36):
+            ignored_names.append('rlimit')
+        if LINUX and get_kernel_version() < (2, 6, 23):
+            ignored_names.append('num_ctx_switches')
         for name in dir(psutil.Process):
-            if name.startswith('_') \
-            or name.startswith('set_') \
-            or name in ('terminate', 'kill', 'suspend', 'resume', 'nice',
-                        'send_signal', 'wait', 'get_children', 'as_dict'):
+            if (name.startswith('_') or name in ignored_names):
                 continue
             else:
                 try:
-                    num1 = p.get_num_fds()
+                    num1 = p.num_fds()
                     for x in range(2):
                         call(p, name)
-                    num2 = p.get_num_fds()
+                    num2 = p.num_fds()
                 except psutil.AccessDenied:
                     pass
                 else:
@@ -228,13 +262,19 @@ class PosixSpecificTestCase(unittest.TestCase):
         if failures:
             self.fail('\n' + '\n'.join(failures))
 
+    @unittest.skipUnless(os.path.islink("/proc/%s/cwd" % os.getpid()),
+                         "/proc fs not available")
+    def test_cwd_proc(self):
+        self.assertEqual(os.readlink("/proc/%s/cwd" % os.getpid()),
+                         psutil.Process().cwd())
 
-def test_main():
+
+def main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(PosixSpecificTestCase))
     result = unittest.TextTestRunner(verbosity=2).run(test_suite)
     return result.wasSuccessful()
 
 if __name__ == '__main__':
-    if not test_main():
+    if not main():
         sys.exit(1)
